/*
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
 * 
 * Contact Information:
 * Dambach-Werke GmbH
 * Elektronische Leitsysteme
 * Fritz-Minhardt-Str. 1
 * 76456 Kuppenheim
 * Phone: +49-7222-402-0
 * Fax: +49-7222-402-200
 * mailto: info@els.dambach.de
 */

package de.bsvrz.vew.sim.sim.steuerung;

import java.util.Iterator;
import java.util.Map;

import de.bsvrz.dav.daf.main.ClientDavInterface;
import de.bsvrz.dav.daf.main.Data;
import de.bsvrz.dav.daf.main.Data.Array;
import de.bsvrz.dav.daf.main.Data.NumberValue;
import de.bsvrz.dav.daf.main.Data.ReferenceValue;
import de.bsvrz.dav.daf.main.Data.TextValue;
import de.bsvrz.dav.daf.main.config.Aspect;
import de.bsvrz.dav.daf.main.config.AttributeGroup;
import de.bsvrz.dav.daf.main.config.SystemObject;
import de.bsvrz.sys.funclib.debug.Debug;
import de.bsvrz.vew.sim.sim.sender.Sender;
import de.bsvrz.vew.sim.sim.simulation.Simulation;
import de.bsvrz.vew.sim.sim.simulationsstrecke.SimulationsStrecke;

/**
 * Starte die Applikationen einer Simulationsstrecke 
 * mit der SWE StartStopp und verarbeite Quittungen
 *  
 * @version  $Revision: 1.1 $ / $Date: 2008/10/31 13:30:30 $ / ($Author: Sans $)
 * @author  Dambach-Werke GmbH
 * @author  Stefan Sans
 */
public class StartStopStart extends StartInfoVerwaltung 
{
  /**
   * Debug-Logger fr Logging-Ausgaben
   */
  private static final Debug _debug = Debug.getLogger();
  
  /**
   * Erzeuge neue Instanz fr Simulation
   * 
   * @param simulation die zugeordnete Simulation
   */
  public StartStopStart(Simulation simulation)
  {
    super(simulation);
  }
  
  /**
   * Starte alle Applikationen der Simulationsstrecke der zugeordneten Simulation
   * 
   * @return true, wenn erfolgreich, false sonst
   */
  public boolean sendeStart()
  {
    boolean result = true;
    SimulationsStrecke ss = _simulation.getSimulationsStrecke();
    Map<String, String> skripte = ss.getStartSkriptInfo();
    
    if (skripte.size() > 0)
    {
      ClientDavInterface con = _simulation.getConnection();
      AttributeGroup atg = con.getDataModel().getAttributeGroup("atg.startStoppAnfrage");
      Aspect asp = con.getDataModel().getAspect("asp.anfrage");
      short sv = 0;
      
      int anzahlSendungen = 0;
      String simPid = _simulation.getObjekt().getPid();
      StartStopQuittierung quittierung = new StartQuittierung();
      quittierung.anmelden();
      
      Map<String, String> startInfo = getStartInfo();
          
      Iterator it = skripte.keySet().iterator();
      
      while (result && it.hasNext())
      {
        String pid = (String)it.next();
        SystemObject rechner = con.getDataModel().getObject(pid);
        if (rechner != null && rechner.isOfType("typ.rechner"))
        {
          Data data = con.createData(atg);
          
          ReferenceValue applRef = data.getReferenceValue("absenderId");
          applRef.setSystemObject(con.getLocalApplicationObject());
          
          TextValue textValue = data.getTextValue("absenderZeichen");
          textValue.setText(simPid + "\\" + pid + "+");
          
          Array array = data.getArray("V3");
          array.setLength(1);
          Data datensatz = array.getItem(0);
          
          NumberValue nv = datensatz.getUnscaledValue("SimulationsVariante");
          nv.set(_simulation.getSimulationsVariante());
          
          String startId = startInfo.get(pid);
          
          if (startId != null)
          {
             // Auf diesem Rechner luft bereits ein Startskript
             // oder dieses ist gestoppt
             // Es wird neu gestartet
             
            textValue = datensatz.getTextValue("StartStoppID");
            textValue.setText(startId);
            textValue = datensatz.getTextValue("xmlDatei");
            textValue.setText("");
            textValue = datensatz.getTextValue("Aktion");
            textValue.setText("neustart");
            _debug.config("Auf Rechner " + pid + 
                " wird das bereits laufende Startskript mit der ID '" +  
                datensatz.getTextValue("StartStoppID") + "' fr Simulation " + 
                _simulation + " neu gestartet");
          }
          else
          {
            textValue = datensatz.getTextValue("StartStoppID");
            textValue.setText("");
            textValue = datensatz.getTextValue("xmlDatei");
            textValue.setText((String)skripte.get(pid));
            textValue = datensatz.getTextValue("Aktion");
            textValue.setText("starten");
            _debug.config("Auf Rechner " + pid + " wird das Startskript '" + 
                datensatz.getTextValue("xmlDatei") + "' fr Simulation " + 
                _simulation + " " + "initial gestartet");
          }
        
          if (!Sender.send(con, rechner, atg, asp, sv, data))
          {
            _debug.error("Fehler beim Senden an Rechnerobjekt " + rechner);
            result = false;
          }
          else
          {  
            anzahlSendungen += 1;
            _debug.fine("Start-Anforderung fr Simulation " + 
                _simulation + "erfolgreich versendet");
          }  
        }
        else
        {
          _debug.error("Objekt "+ rechner + " ist als Rechnerobjekt ungltig");
          result = false;
        }
      }
      
      if (anzahlSendungen > 0)
      {  
        _debug.fine("Warte max. " + StartStopQuittierung.QUITTUNGS_TIMEOUT + 
            "ms auf " + anzahlSendungen + " Start-Quittierung(en) fr Simulation " + 
            simPid);
        
        quittierung.warteAufQuittierungen(anzahlSendungen);
                
        /*
        long aktZeit = System.currentTimeMillis();
        boolean quittierungVollstaendig = false;
        while (!quittierungVollstaendig &&
            System.currentTimeMillis() - aktZeit < Quittierung.QUITTUNGS_TIMEOUT)
        {
          quittierungVollstaendig = (anzahlSendungen == quittierung.getAnzahlQuittierungen());
        }
        */
        
        if (quittierung.getAnzahlQuittierungen() < anzahlSendungen)
        {
          _debug.error("Starten der Start-Skripte fr Simulation " + simPid +
              " wurde nicht vollstndig quittiert (Timeout)");
          result = false; 
        }
        else if (quittierung.getPosQuittierung().size() < anzahlSendungen)
        {
          _debug.error("Starten der Start-Skripte fr Simulation " + simPid +
              " wurde negativ quittiert");
          result = false; 
        }
        else if (result)
        {
          _debug.info("Starten der Start-Skripte fr Simulation " + simPid +
          " wurde vollstndig und positiv quittiert");
        }
      }
      
      quittierung.abmelden();
      startInfo.putAll(quittierung.getPosQuittierung());
      aktualisiereStartInfo(startInfo);
    }
    else
    {
      _debug.error("Simulationsstrecke " + ss + " hat leere Startskript-Liste");
      result = false;
    }
    
    return result;
  }
  
  /**
   * Bearbeite Start-Quittierungen von StartStopp
   * 
   * @author Stefan Sans
   */
  class StartQuittierung extends StartStopQuittierung
  {
    /**
     * Default-Konstruktor
     */
    StartQuittierung()
    {
      super(_simulation.getObjekt().getPid(), "+");
    }
    
    @Override
    public ClientDavInterface getConnection()
    {
      return _simulation.getConnection();
    }
    
    @Override
    protected boolean istQuittierungPositiv(Data data)
    {
      // int zustand = data.getItem("zustand").asUnscaledValue().intValue();
      // int fehler = data.getItem("fehler").asUnscaledValue().intValue();
      
      String zustandText = data.getItem("zustand").asTextValue().getText();
      String fehlerText = data.getItem("fehler").asTextValue().getText();
      
      if (zustandText.equals("gestartet") && fehlerText.equals("kein Fehler"))
      {
        return true;
      }
      
      return false;
    }
  }
}
